'***************************************************************
'**                                                           **
'**  Program: compute.bas  (QUICKBASIC 4.5)                   **
'**  Written:  2-04-91 By Milton Watts                        **
'**  Updates:                                                 **
'**                                                           **
'**  COPYRIGHT 1990, 1991 QUARTZDYNE, Inc.                    **
'***************************************************************
' allow dynamic allocation of variable space
' $DYNAMIC

DEFINT I-N
DEFDBL A-H, O-Z
OPTION BASE 1

TYPE PRESCALE    ' prescale constants and fit order (2 per transducer)
  n AS INTEGER
  p AS INTEGER
  m AS DOUBLE
  f0 AS DOUBLE
END TYPE

DECLARE FUNCTION readcffile (id$, label$, unit$, ps1 AS PRESCALE, ps2 AS PRESCALE, c() AS DOUBLE, span AS DOUBLE, zero AS DOUBLE)
DECLARE FUNCTION calculate (f1#, f2#, ps1 AS ANY, PRESCALE AS PRESCALE, c#(), span AS DOUBLE, zero AS DOUBLE)
DECLARE FUNCTION dospath$ (file$)

' user defined functions for byte manipulation

CONST false = 0
CONST true = NOT false

DIM ps1 AS PRESCALE          ' prescale for f1
DIM ps2 AS PRESCALE          ' prescale for f2
DIM c(1) AS DOUBLE           ' coefficient array (redim to appropriate size)

file$ = COMMAND$             ' get coefficient file to load from command line
IF file$ = "" THEN           ' if not in command line . . .
  IF ENVIRON$("CFFILE") <> "" THEN    ' check environment string CFFILE
    file$ = ENVIRON$("CFFILE")        ' from DOS:  >SET CFFILE=cffilepath
  END IF
END IF

retry:
IF readcffile(id$, label$, unit$, ps1, ps2, c(), span, zero) THEN ' read coefficient file
  IF (ps1.n <= 0 AND ps2.n <= 0) THEN
    PRINT "Coefficient file format error"
    GOTO ErrExit
  END IF
  PRINT
  PRINT "Transducer ID: "; id$
  PRINT
  DO
    IF ps1.n > 0 THEN
      INPUT "Enter Pressure Frequency    >", fc1.freq
      IF fc1.freq = 0 GOTO ErrExit
    END IF
    IF ps2.n > 0 THEN
      INPUT "Enter Temperature Frequency >", fc2.freq
      IF fc2.freq = 0 GOTO ErrExit
    END IF
    pressure = calculate(fc1.freq, fc2.freq, ps1, ps2, c(), span, zero)
    PRINT TAB(20); label$; " = "; pressure; unit$
  LOOP
END IF

ErrExit: PRINT "End of program"
CLOSE #1
END

fileError:
  PRINT "Unable to open file "; file$
  RESUME ErrExit

dataerror:
  PRINT "Data not in correct format"
  RESUME ErrExit

diskError:
  PRINT "Coefficient file not found in drive "; file$
  BEEP
  INPUT "Enter new path name (NULL to cancel)"; a$
  IF a$ = "" THEN RESUME ErrExit
  file$ = a$
  RESUME retry

REM $STATIC
FUNCTION calculate (f1, f2, ps1 AS PRESCALE, ps2 AS PRESCALE, c(), span AS DOUBLE, zero AS DOUBLE)
  x1 = ps1.m * (f1 - ps1.f0)      ' prescale pressure frequency
  x2 = ps2.m * (f2 - ps2.f0)      ' prescale temperature frequency
  i = (ps1.n + 1) * (ps2.n + 1)   ' total number of coefficients
  z = 0#
  FOR j = 0 TO ps1.n              ' fit order in pressure frequency
    z1 = c(i)                     ' z1 becomes temp compensated pres coeff
    i = i - 1                     ' count down coefficients as used
    FOR k = 1 TO ps2.n            ' fit order in temperature frequency
      z1 = x2 * z1 + c(i)         ' polynomial in x2
      i = i - 1
    NEXT k
    z = x1 * z + z1               ' polynomial in x1
  NEXT j
  calculate = z * span + zero     ' return computed pressure
END FUNCTION

FUNCTION dospath$ (file$)        ' return portion of file$ that is the
  i = INSTR(file$, ":")          ' DOS path specification
  DO
    j = INSTR(i + 1, file$, "\")
    IF j = 0 THEN EXIT DO
    i = j
  LOOP
  dospath$ = LEFT$(file$, i)
END FUNCTION

FUNCTION readcffile (id$, label$, unit$, ps1 AS PRESCALE, ps2 AS PRESCALE, c() AS DOUBLE, span AS DOUBLE, zero AS DOUBLE)
  SHARED file$
  IF file$ = "" THEN file$ = "*.cff"
  IF INSTR(file$, ".") = 0 THEN file$ = file$ + ".cff"
  DO
    IF INSTR(file$, "*") = 0 AND INSTR(file$, "?") = 0 THEN EXIT DO
    drive$ = dospath$(file$)
    PRINT "DIR "; file$
    ON ERROR GOTO diskError
    FILES file$
    ON ERROR GOTO 0
    PRINT
    INPUT "Enter file name"; file$
    IF INSTR(file$, ".") = 0 THEN file$ = file$ + ".CFF"
    IF dospath$(file$) = "" THEN file$ = drive$ + file$
  LOOP    ' in readcffile
  ON ERROR GOTO fileError
  OPEN "I", #1, file$

  ON ERROR GOTO dataerror
  IF EOF(1) GOTO fileEnd
  INPUT #1, id$, label$, unit$           ' file header info
  IF EOF(1) GOTO fileEnd
  INPUT #1, ps2.n, ps2.p, ps2.m, ps2.f0  ' temperature fit order and prescale
  IF EOF(1) GOTO fileEnd
  INPUT #1, ps1.n, ps1.p, ps1.m, ps1.f0  ' pressure fit order and prescale
  n = (ps1.n + 1) * (ps2.n + 1)          ' total number of coefficients
  ON ERROR GOTO 0
  REDIM c(n) AS DOUBLE
  FOR i = 1 TO n
    IF EOF(1) THEN GOTO fileEnd
    INPUT #1, c(i)                       ' coefficients in order
  NEXT i
  IF EOF(1) GOTO noSpan
  INPUT #1, span, zero
  ON ERROR GOTO 0
  CLOSE #1
  readcffile = true                      ' succesfully read
  EXIT FUNCTION

noSpan:                                  ' old cf file format - no span/zero
  span = 1#
  zero = 0#
  ON ERROR GOTO 0
  CLOSE #1
  readcffile = true
  EXIT FUNCTION

fileEnd:
  ON ERROR GOTO 0
  CLOSE #1
  readcffile = false                     ' unable to read cf file
END FUNCTION

